// gcboost.h
#ifndef GCBOOST_H
#define GCBOOST_H
#include "../blmap.h"

class CGCBoost:public CBLContext
{
public:
	typedef int		(CGCBoost::*PF_I_IV)(int,CValue&);			// GetPropVal,SetPropVal
	typedef int		(CGCBoost::*PF_I_I) (int);					// IsPropReadable,IsPropWritable,GetNParams,HasRetVal
	typedef int		(CGCBoost::*PF_I_IIV)(int,int,CValue *);	// GetParamDefValue
	typedef int		(CGCBoost::*PF_CAP)(int,CValue **);			// CallAsProc
	typedef int		(CGCBoost::*PF_CAF)(int,CValue&,CValue**);	// CallAsFunc
	typedef char const*(CGCBoost::*PF_C_II)(int,int);		// GetPropName,GetMethodName

	struct SMethInfo
	{
		int HasRetVal;
		int NumberOfParam;
	};
	struct BatchGuard
	{
		BOOL bNeedEndBatch;
		CGetDoc7* pDoc;
		BatchGuard(CGetDoc7* pd):pDoc(pd), bNeedEndBatch(FALSE)
		{
			if(pDoc)
			{
				bNeedEndBatch=!pDoc->IsBatchStarted();
				pDoc->OnStartBatch();
			}
		}
		~BatchGuard(){if(bNeedEndBatch) pDoc->OnEndBatch(TRUE);}
	};
	struct SGCData{
		SGCData(CBLModule7* pMod,CBLContext* pCont,BOOL optimize)
		{
			m_nMethInCtx	= pCont->GetNMethods();
			DWORD n			= pMod->GetNProcs();
			m_nMethAll		= m_nMethInCtx + n - 1;
			m_MethInfo		= NULL;
			
			if(m_nMethAll)
				m_MethInfo=new SMethInfo[m_nMethAll];
			for(DWORD i=0;i<m_nMethInCtx;i++)
			{
				m_MethMap.InsertKey(pCont->GetMethodName(i, 0), i);
				m_MethMap.InsertKey(pCont->GetMethodName(i, 1), i);
				m_MethInfo[i].NumberOfParam=pCont->GetNParams(i);
				m_MethInfo[i].HasRetVal=pCont->HasRetVal(i);
			}
			for(DWORD j=1; j<n; j++, i++)
			{
				CBLProcInfo info;
				pMod->GetProcInfo(j,info);
				m_MethMap.InsertKey(info.GetName(), i);
				m_MethInfo[i].NumberOfParam=info.GetNParams();
				m_MethInfo[i].HasRetVal=info.HasRetVal();
			}
			m_MethMap.Optimize();
			//   
			m_nPropInCtx = pCont->GetNProps();
			for(i=0; i<m_nPropInCtx; i++)
			{
				m_PropMap.InsertKey(pCont->GetPropName(i, 0), i);
				m_PropMap.InsertKey(pCont->GetPropName(i, 1), i);
			}
			n = pMod->GetNStaticVars();
			m_nPropAll=m_nPropInCtx + n;
			for(j=0; j<n; j++,i++)
			{
				CBLVarInfo info;
				pMod->GetStaticVarDescr(j,info);
				m_PropMap.InsertKey(info.GetName(),i);
			}
			m_PropMap.Optimize();
			m_Destroy=!optimize;
			if(optimize)
				m_DataMap[pMod->GetFullName()]=this;
		}
		
		~SGCData(){delete m_MethInfo;}

		void Remove()
		{
			if(m_Destroy)
				delete this;
		}
		static SGCData* GetData(CBLModule7* pMod,CBLContext* pCont,BOOL optimize)
		{
			SGCData* pRet=NULL;
			if(optimize)
				m_DataMap.Lookup(pMod->GetFullName(), (void*&)pRet);
			if(!pRet)
				pRet=new SGCData(pMod,pCont,optimize);
			return pRet;
		}

		int	FindProp(char const*name)const
		{
			return m_PropMap.GetKey(name);
		}
		int GetPropVal(const CGCBoost* pCont, DWORD dwPropNum, CValue &val)
		{
			if(dwPropNum >= m_nPropInCtx)
				return pCont->GetModule()->GetStaticVarValue(dwPropNum-m_nPropInCtx, val, 0);
			else
				return (pCont->*(*(PF_I_IV*)&m_pRealVTable[21]))(dwPropNum, val);
		}
		int SetPropVal(CGCBoost* pCont,DWORD dwPropNum, CValue const &val)
		{
			if(dwPropNum>=m_nPropInCtx)
				return pCont->GetModule()->SetStaticVarValue(dwPropNum-m_nPropInCtx,val,0);
			else
				return (pCont->*(*(PF_I_IV*)&m_pRealVTable[22]))(dwPropNum, const_cast<CValue&>(val));
		}
		int IsPropReadable(const CGCBoost* pCont,DWORD dwPropNum)
		{
			if(dwPropNum>=m_nPropInCtx)
				return 1;
			else
				return (pCont->*(*(PF_I_I*)&m_pRealVTable[23]))(dwPropNum);
		}
		int IsPropWritable(const CGCBoost* pCont,DWORD dwPropNum)
		{
			if(dwPropNum >= m_nPropInCtx)
				return 1;
			else
				return (pCont->*(*(PF_I_I*)&m_pRealVTable[24]))(dwPropNum);
		}
		int FindMethod(char const *name)
		{
			return m_MethMap.GetKey(name);
		}

		int GetNMethods()
		{
			return m_nMethAll;
		}
		int GetNProps()
		{
			return m_nPropAll;
		}
		char const * GetMethodName(const CGCBoost* pCont, DWORD dwMethNum, DWORD dwAlias)const
		{
			if(dwMethNum < m_nMethInCtx)
				return (pCont->*(*(PF_C_II*)&m_pRealVTable[27]))(dwMethNum, dwAlias);
			if(!dwAlias)
			{
				CBLProcInfo info;
				pCont->GetModule()->GetProcInfo(dwMethNum - m_nMethInCtx + 1, info);
				return info.GetName();
			}
			return NULL;
		}
		char const * GetPropName(const CGCBoost* pCont, DWORD dwPropNum, DWORD dwAlias)const
		{
			if(dwPropNum < m_nPropInCtx)
				return (pCont->*(*(PF_C_II*)&m_pRealVTable[20]))(dwPropNum, dwAlias);
			if(!dwAlias)
			{
				CBLVarInfo info;
				pCont->GetModule()->GetStaticVarDescr(dwPropNum - m_nPropInCtx, info);
				return info.GetName();
			}
			return NULL;

		}
		int GetNParams(int iMethNum)
		{
			return m_MethInfo[iMethNum].NumberOfParam;
		}
		int GetParamDefValue(const CGCBoost* pCont,int iMethNum,int iParamNum,class CValue *pVal)
		{
			if(iMethNum >= m_nMethInCtx)
				return 1;
			return (pCont->*(*(PF_I_IIV*)&m_pRealVTable[29]))(iMethNum,iParamNum,pVal);
		}
		int HasRetVal(int iMethNum)
		{
			return m_MethInfo[iMethNum].HasRetVal;
		}
		int CallAsProc(CGCBoost* pCont,int iMethNum,CValue **ppParam)
		{
			if(iMethNum >= m_nMethInCtx)
			{
				BatchGuard bd(pCont->GetDoc7());
				return pCont->GetModule()->CallAsProc(iMethNum-m_nMethInCtx + 1, m_MethInfo[iMethNum].NumberOfParam,ppParam);
			}
			return (pCont->*(*(PF_CAP*)&m_pRealVTable[31]))(iMethNum,ppParam);
		}
		int CallAsFunc(CGCBoost* pCont,int iMethNum,CValue& retVal,CValue **ppParam)
		{
			if(iMethNum >= m_nMethInCtx)
			{
				CValue locVal;
				BatchGuard bd(pCont->GetDoc7());
				int ret=pCont->GetModule()->CallAsFunc(iMethNum-m_nMethInCtx + 1, locVal,m_MethInfo[iMethNum].NumberOfParam,ppParam);
				retVal=locVal;
				return ret;
			}
			return (pCont->*(*(PF_CAF*)&m_pRealVTable[32]))(iMethNum,retVal,ppParam);
		}

		BOOL	m_Destroy;
		CBLMap	m_MethMap;
		DWORD	m_nMethInCtx;
		DWORD	m_nMethAll;
		CBLMap	m_PropMap;
		DWORD	m_nPropInCtx;
		DWORD	m_nPropAll;
		SMethInfo* m_MethInfo;
		static CMapStringToPtr m_DataMap;
	};
	struct SGCInfo
	{
		SGCInfo(CBLModule* pMod, SGCData* pData, CGetDoc7* pDoc=NULL):m_pMod(pMod), m_pData(pData), m_pDoc(pDoc){}
		~SGCInfo(){m_pData->Remove();}
		CBLModule* m_pMod;				// -      
		CGetDoc7*  m_pDoc;
		SGCData*   m_pData;
	};
	virtual  ~CGCBoost(void);												//1
	virtual int				GetNProps(void)const;							//18 // artbear
	virtual int				FindProp(char const *)const;					//19
	virtual char const *	GetPropName(int,int)const;						//20 // artbear
	virtual int				GetPropVal(int,class CValue &)const;			//21
	virtual int				SetPropVal(int,class CValue const &);			//22
	virtual int				IsPropReadable(int)const;						//23
	virtual int				IsPropWritable(int)const;						//24
	virtual int				GetNMethods(void)const;							//25 // artbear
	virtual int				FindMethod(char const *)const;					//26
	virtual char const *	GetMethodName(int,int)const;					//27 // artbear
	virtual int				GetNParams(int)const;							//28
	virtual int				GetParamDefValue(int,int,class CValue *)const;	//29
	virtual int				HasRetVal(int)const;							//30
	virtual int				CallAsProc(int,class CValue * *); 				//31
	virtual int				CallAsFunc(int,class CValue &,class CValue * *);//32

	static void Init();
	static BOOL Boost(CBLModule7 *pMod,BOOL optimize);
	int AssignAndCompile(CBLModule7*);
	int WrapCompile();
	void GroupDestruct();
	
	SGCInfo*	GetInfo()	const	{return (SGCInfo*)m_FlagAutoDestroy;}
	CBLModule*	GetModule()	const	{return GetInfo()->m_pMod;}
	SGCData*	GetData()	const	{return GetInfo()->m_pData;}
	CGetDoc7*	GetDoc7()	const	{return GetInfo()->m_pDoc;}
	
	static DWORD* m_pMyVTable;
	static DWORD* m_pRealVTable;
};

#endif